# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['SynchronizationJobArgs', 'SynchronizationJob']

@pulumi.input_type
class SynchronizationJobArgs:
    def __init__(__self__, *,
                 service_principal_id: pulumi.Input[str],
                 template_id: pulumi.Input[str],
                 enabled: Optional[pulumi.Input[bool]] = None):
        """
        The set of arguments for constructing a SynchronizationJob resource.
        :param pulumi.Input[str] service_principal_id: The object ID of the service principal for which this synchronization job should be created. Changing this field forces a new resource to be created.
        :param pulumi.Input[str] template_id: Identifier of the synchronization template this job is based on.
        :param pulumi.Input[bool] enabled: Whether or not the provisioning job is enabled. Default state is `true`.
        """
        SynchronizationJobArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            service_principal_id=service_principal_id,
            template_id=template_id,
            enabled=enabled,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             service_principal_id: pulumi.Input[str],
             template_id: pulumi.Input[str],
             enabled: Optional[pulumi.Input[bool]] = None,
             opts: Optional[pulumi.ResourceOptions]=None):
        _setter("service_principal_id", service_principal_id)
        _setter("template_id", template_id)
        if enabled is not None:
            _setter("enabled", enabled)

    @property
    @pulumi.getter(name="servicePrincipalId")
    def service_principal_id(self) -> pulumi.Input[str]:
        """
        The object ID of the service principal for which this synchronization job should be created. Changing this field forces a new resource to be created.
        """
        return pulumi.get(self, "service_principal_id")

    @service_principal_id.setter
    def service_principal_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "service_principal_id", value)

    @property
    @pulumi.getter(name="templateId")
    def template_id(self) -> pulumi.Input[str]:
        """
        Identifier of the synchronization template this job is based on.
        """
        return pulumi.get(self, "template_id")

    @template_id.setter
    def template_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "template_id", value)

    @property
    @pulumi.getter
    def enabled(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether or not the provisioning job is enabled. Default state is `true`.
        """
        return pulumi.get(self, "enabled")

    @enabled.setter
    def enabled(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "enabled", value)


@pulumi.input_type
class _SynchronizationJobState:
    def __init__(__self__, *,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 schedules: Optional[pulumi.Input[Sequence[pulumi.Input['SynchronizationJobScheduleArgs']]]] = None,
                 service_principal_id: Optional[pulumi.Input[str]] = None,
                 template_id: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering SynchronizationJob resources.
        :param pulumi.Input[bool] enabled: Whether or not the provisioning job is enabled. Default state is `true`.
        :param pulumi.Input[Sequence[pulumi.Input['SynchronizationJobScheduleArgs']]] schedules: A `schedule` list as documented below.
        :param pulumi.Input[str] service_principal_id: The object ID of the service principal for which this synchronization job should be created. Changing this field forces a new resource to be created.
        :param pulumi.Input[str] template_id: Identifier of the synchronization template this job is based on.
        """
        _SynchronizationJobState._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            enabled=enabled,
            schedules=schedules,
            service_principal_id=service_principal_id,
            template_id=template_id,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             enabled: Optional[pulumi.Input[bool]] = None,
             schedules: Optional[pulumi.Input[Sequence[pulumi.Input['SynchronizationJobScheduleArgs']]]] = None,
             service_principal_id: Optional[pulumi.Input[str]] = None,
             template_id: Optional[pulumi.Input[str]] = None,
             opts: Optional[pulumi.ResourceOptions]=None):
        if enabled is not None:
            _setter("enabled", enabled)
        if schedules is not None:
            _setter("schedules", schedules)
        if service_principal_id is not None:
            _setter("service_principal_id", service_principal_id)
        if template_id is not None:
            _setter("template_id", template_id)

    @property
    @pulumi.getter
    def enabled(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether or not the provisioning job is enabled. Default state is `true`.
        """
        return pulumi.get(self, "enabled")

    @enabled.setter
    def enabled(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "enabled", value)

    @property
    @pulumi.getter
    def schedules(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['SynchronizationJobScheduleArgs']]]]:
        """
        A `schedule` list as documented below.
        """
        return pulumi.get(self, "schedules")

    @schedules.setter
    def schedules(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['SynchronizationJobScheduleArgs']]]]):
        pulumi.set(self, "schedules", value)

    @property
    @pulumi.getter(name="servicePrincipalId")
    def service_principal_id(self) -> Optional[pulumi.Input[str]]:
        """
        The object ID of the service principal for which this synchronization job should be created. Changing this field forces a new resource to be created.
        """
        return pulumi.get(self, "service_principal_id")

    @service_principal_id.setter
    def service_principal_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "service_principal_id", value)

    @property
    @pulumi.getter(name="templateId")
    def template_id(self) -> Optional[pulumi.Input[str]]:
        """
        Identifier of the synchronization template this job is based on.
        """
        return pulumi.get(self, "template_id")

    @template_id.setter
    def template_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "template_id", value)


class SynchronizationJob(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 service_principal_id: Optional[pulumi.Input[str]] = None,
                 template_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Manages a synchronization job associated with a service principal (enterprise application) within Azure Active Directory.

        ## API Permissions

        The following API permissions are required in order to use this resource.

        When authenticated with a service principal, this resource requires one of the following application roles: `Application.ReadWrite.All` or `Directory.ReadWrite.All`

        ## Example Usage

        *Basic example*

        ```python
        import pulumi
        import pulumi_azuread as azuread

        example_application_template = azuread.get_application_template(display_name="Azure Databricks SCIM Provisioning Connector")
        example_application = azuread.Application("exampleApplication",
            display_name="example",
            template_id=example_application_template.template_id,
            feature_tags=[azuread.ApplicationFeatureTagArgs(
                enterprise=True,
                gallery=True,
            )])
        example_service_principal = azuread.ServicePrincipal("exampleServicePrincipal",
            application_id=example_application.application_id,
            use_existing=True)
        example_synchronization_secret = azuread.SynchronizationSecret("exampleSynchronizationSecret",
            service_principal_id=example_service_principal.id,
            credentials=[
                azuread.SynchronizationSecretCredentialArgs(
                    key="BaseAddress",
                    value="https://adb-example.azuredatabricks.net/api/2.0/preview/scim",
                ),
                azuread.SynchronizationSecretCredentialArgs(
                    key="SecretToken",
                    value="some-token",
                ),
            ])
        example_synchronization_job = azuread.SynchronizationJob("exampleSynchronizationJob",
            service_principal_id=example_service_principal.id,
            template_id="dataBricks",
            enabled=True)
        ```

        ## Import

        Synchronization jobs can be imported using the `id`, e.g.

        ```sh
         $ pulumi import azuread:index/synchronizationJob:SynchronizationJob example 00000000-0000-0000-0000-000000000000/job/dataBricks.f5532fc709734b1a90e8a1fa9fd03a82.8442fd39-2183-419c-8732-74b6ce866bd5
        ```

         -> This ID format is unique to Terraform and is composed of the Service Principal Object ID and the ID of the Synchronization Job Id in the format `{servicePrincipalId}/job/{jobId}`.

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] enabled: Whether or not the provisioning job is enabled. Default state is `true`.
        :param pulumi.Input[str] service_principal_id: The object ID of the service principal for which this synchronization job should be created. Changing this field forces a new resource to be created.
        :param pulumi.Input[str] template_id: Identifier of the synchronization template this job is based on.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: SynchronizationJobArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages a synchronization job associated with a service principal (enterprise application) within Azure Active Directory.

        ## API Permissions

        The following API permissions are required in order to use this resource.

        When authenticated with a service principal, this resource requires one of the following application roles: `Application.ReadWrite.All` or `Directory.ReadWrite.All`

        ## Example Usage

        *Basic example*

        ```python
        import pulumi
        import pulumi_azuread as azuread

        example_application_template = azuread.get_application_template(display_name="Azure Databricks SCIM Provisioning Connector")
        example_application = azuread.Application("exampleApplication",
            display_name="example",
            template_id=example_application_template.template_id,
            feature_tags=[azuread.ApplicationFeatureTagArgs(
                enterprise=True,
                gallery=True,
            )])
        example_service_principal = azuread.ServicePrincipal("exampleServicePrincipal",
            application_id=example_application.application_id,
            use_existing=True)
        example_synchronization_secret = azuread.SynchronizationSecret("exampleSynchronizationSecret",
            service_principal_id=example_service_principal.id,
            credentials=[
                azuread.SynchronizationSecretCredentialArgs(
                    key="BaseAddress",
                    value="https://adb-example.azuredatabricks.net/api/2.0/preview/scim",
                ),
                azuread.SynchronizationSecretCredentialArgs(
                    key="SecretToken",
                    value="some-token",
                ),
            ])
        example_synchronization_job = azuread.SynchronizationJob("exampleSynchronizationJob",
            service_principal_id=example_service_principal.id,
            template_id="dataBricks",
            enabled=True)
        ```

        ## Import

        Synchronization jobs can be imported using the `id`, e.g.

        ```sh
         $ pulumi import azuread:index/synchronizationJob:SynchronizationJob example 00000000-0000-0000-0000-000000000000/job/dataBricks.f5532fc709734b1a90e8a1fa9fd03a82.8442fd39-2183-419c-8732-74b6ce866bd5
        ```

         -> This ID format is unique to Terraform and is composed of the Service Principal Object ID and the ID of the Synchronization Job Id in the format `{servicePrincipalId}/job/{jobId}`.

        :param str resource_name: The name of the resource.
        :param SynchronizationJobArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(SynchronizationJobArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            kwargs = kwargs or {}
            def _setter(key, value):
                kwargs[key] = value
            SynchronizationJobArgs._configure(_setter, **kwargs)
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 service_principal_id: Optional[pulumi.Input[str]] = None,
                 template_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = SynchronizationJobArgs.__new__(SynchronizationJobArgs)

            __props__.__dict__["enabled"] = enabled
            if service_principal_id is None and not opts.urn:
                raise TypeError("Missing required property 'service_principal_id'")
            __props__.__dict__["service_principal_id"] = service_principal_id
            if template_id is None and not opts.urn:
                raise TypeError("Missing required property 'template_id'")
            __props__.__dict__["template_id"] = template_id
            __props__.__dict__["schedules"] = None
        super(SynchronizationJob, __self__).__init__(
            'azuread:index/synchronizationJob:SynchronizationJob',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            enabled: Optional[pulumi.Input[bool]] = None,
            schedules: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['SynchronizationJobScheduleArgs']]]]] = None,
            service_principal_id: Optional[pulumi.Input[str]] = None,
            template_id: Optional[pulumi.Input[str]] = None) -> 'SynchronizationJob':
        """
        Get an existing SynchronizationJob resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] enabled: Whether or not the provisioning job is enabled. Default state is `true`.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['SynchronizationJobScheduleArgs']]]] schedules: A `schedule` list as documented below.
        :param pulumi.Input[str] service_principal_id: The object ID of the service principal for which this synchronization job should be created. Changing this field forces a new resource to be created.
        :param pulumi.Input[str] template_id: Identifier of the synchronization template this job is based on.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _SynchronizationJobState.__new__(_SynchronizationJobState)

        __props__.__dict__["enabled"] = enabled
        __props__.__dict__["schedules"] = schedules
        __props__.__dict__["service_principal_id"] = service_principal_id
        __props__.__dict__["template_id"] = template_id
        return SynchronizationJob(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def enabled(self) -> pulumi.Output[Optional[bool]]:
        """
        Whether or not the provisioning job is enabled. Default state is `true`.
        """
        return pulumi.get(self, "enabled")

    @property
    @pulumi.getter
    def schedules(self) -> pulumi.Output[Sequence['outputs.SynchronizationJobSchedule']]:
        """
        A `schedule` list as documented below.
        """
        return pulumi.get(self, "schedules")

    @property
    @pulumi.getter(name="servicePrincipalId")
    def service_principal_id(self) -> pulumi.Output[str]:
        """
        The object ID of the service principal for which this synchronization job should be created. Changing this field forces a new resource to be created.
        """
        return pulumi.get(self, "service_principal_id")

    @property
    @pulumi.getter(name="templateId")
    def template_id(self) -> pulumi.Output[str]:
        """
        Identifier of the synchronization template this job is based on.
        """
        return pulumi.get(self, "template_id")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = ['ServicePrincipalDelegatedPermissionGrantArgs', 'ServicePrincipalDelegatedPermissionGrant']

@pulumi.input_type
class ServicePrincipalDelegatedPermissionGrantArgs:
    def __init__(__self__, *,
                 claim_values: pulumi.Input[Sequence[pulumi.Input[str]]],
                 resource_service_principal_object_id: pulumi.Input[str],
                 service_principal_object_id: pulumi.Input[str],
                 user_object_id: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a ServicePrincipalDelegatedPermissionGrant resource.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] claim_values: A set of claim values for delegated permission scopes which should be included in access tokens for the resource.
        :param pulumi.Input[str] resource_service_principal_object_id: The object ID of the service principal representing the resource to be accessed. Changing this forces a new resource to be created.
        :param pulumi.Input[str] service_principal_object_id: The object ID of the service principal for which this delegated permission grant should be created. Changing this forces a new resource to be created.
        :param pulumi.Input[str] user_object_id: The object ID of the user on behalf of whom the service principal is authorized to access the resource. When omitted, the delegated permission grant will be consented for all users. Changing this forces a new resource to be created.
               
               > **Granting Admin Consent** To grant admin consent for the service principal to impersonate all users, just omit the `user_object_id` property.
        """
        ServicePrincipalDelegatedPermissionGrantArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            claim_values=claim_values,
            resource_service_principal_object_id=resource_service_principal_object_id,
            service_principal_object_id=service_principal_object_id,
            user_object_id=user_object_id,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             claim_values: pulumi.Input[Sequence[pulumi.Input[str]]],
             resource_service_principal_object_id: pulumi.Input[str],
             service_principal_object_id: pulumi.Input[str],
             user_object_id: Optional[pulumi.Input[str]] = None,
             opts: Optional[pulumi.ResourceOptions]=None):
        _setter("claim_values", claim_values)
        _setter("resource_service_principal_object_id", resource_service_principal_object_id)
        _setter("service_principal_object_id", service_principal_object_id)
        if user_object_id is not None:
            _setter("user_object_id", user_object_id)

    @property
    @pulumi.getter(name="claimValues")
    def claim_values(self) -> pulumi.Input[Sequence[pulumi.Input[str]]]:
        """
        A set of claim values for delegated permission scopes which should be included in access tokens for the resource.
        """
        return pulumi.get(self, "claim_values")

    @claim_values.setter
    def claim_values(self, value: pulumi.Input[Sequence[pulumi.Input[str]]]):
        pulumi.set(self, "claim_values", value)

    @property
    @pulumi.getter(name="resourceServicePrincipalObjectId")
    def resource_service_principal_object_id(self) -> pulumi.Input[str]:
        """
        The object ID of the service principal representing the resource to be accessed. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "resource_service_principal_object_id")

    @resource_service_principal_object_id.setter
    def resource_service_principal_object_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "resource_service_principal_object_id", value)

    @property
    @pulumi.getter(name="servicePrincipalObjectId")
    def service_principal_object_id(self) -> pulumi.Input[str]:
        """
        The object ID of the service principal for which this delegated permission grant should be created. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "service_principal_object_id")

    @service_principal_object_id.setter
    def service_principal_object_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "service_principal_object_id", value)

    @property
    @pulumi.getter(name="userObjectId")
    def user_object_id(self) -> Optional[pulumi.Input[str]]:
        """
        The object ID of the user on behalf of whom the service principal is authorized to access the resource. When omitted, the delegated permission grant will be consented for all users. Changing this forces a new resource to be created.

        > **Granting Admin Consent** To grant admin consent for the service principal to impersonate all users, just omit the `user_object_id` property.
        """
        return pulumi.get(self, "user_object_id")

    @user_object_id.setter
    def user_object_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "user_object_id", value)


@pulumi.input_type
class _ServicePrincipalDelegatedPermissionGrantState:
    def __init__(__self__, *,
                 claim_values: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 resource_service_principal_object_id: Optional[pulumi.Input[str]] = None,
                 service_principal_object_id: Optional[pulumi.Input[str]] = None,
                 user_object_id: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering ServicePrincipalDelegatedPermissionGrant resources.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] claim_values: A set of claim values for delegated permission scopes which should be included in access tokens for the resource.
        :param pulumi.Input[str] resource_service_principal_object_id: The object ID of the service principal representing the resource to be accessed. Changing this forces a new resource to be created.
        :param pulumi.Input[str] service_principal_object_id: The object ID of the service principal for which this delegated permission grant should be created. Changing this forces a new resource to be created.
        :param pulumi.Input[str] user_object_id: The object ID of the user on behalf of whom the service principal is authorized to access the resource. When omitted, the delegated permission grant will be consented for all users. Changing this forces a new resource to be created.
               
               > **Granting Admin Consent** To grant admin consent for the service principal to impersonate all users, just omit the `user_object_id` property.
        """
        _ServicePrincipalDelegatedPermissionGrantState._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            claim_values=claim_values,
            resource_service_principal_object_id=resource_service_principal_object_id,
            service_principal_object_id=service_principal_object_id,
            user_object_id=user_object_id,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             claim_values: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
             resource_service_principal_object_id: Optional[pulumi.Input[str]] = None,
             service_principal_object_id: Optional[pulumi.Input[str]] = None,
             user_object_id: Optional[pulumi.Input[str]] = None,
             opts: Optional[pulumi.ResourceOptions]=None):
        if claim_values is not None:
            _setter("claim_values", claim_values)
        if resource_service_principal_object_id is not None:
            _setter("resource_service_principal_object_id", resource_service_principal_object_id)
        if service_principal_object_id is not None:
            _setter("service_principal_object_id", service_principal_object_id)
        if user_object_id is not None:
            _setter("user_object_id", user_object_id)

    @property
    @pulumi.getter(name="claimValues")
    def claim_values(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        A set of claim values for delegated permission scopes which should be included in access tokens for the resource.
        """
        return pulumi.get(self, "claim_values")

    @claim_values.setter
    def claim_values(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "claim_values", value)

    @property
    @pulumi.getter(name="resourceServicePrincipalObjectId")
    def resource_service_principal_object_id(self) -> Optional[pulumi.Input[str]]:
        """
        The object ID of the service principal representing the resource to be accessed. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "resource_service_principal_object_id")

    @resource_service_principal_object_id.setter
    def resource_service_principal_object_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "resource_service_principal_object_id", value)

    @property
    @pulumi.getter(name="servicePrincipalObjectId")
    def service_principal_object_id(self) -> Optional[pulumi.Input[str]]:
        """
        The object ID of the service principal for which this delegated permission grant should be created. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "service_principal_object_id")

    @service_principal_object_id.setter
    def service_principal_object_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "service_principal_object_id", value)

    @property
    @pulumi.getter(name="userObjectId")
    def user_object_id(self) -> Optional[pulumi.Input[str]]:
        """
        The object ID of the user on behalf of whom the service principal is authorized to access the resource. When omitted, the delegated permission grant will be consented for all users. Changing this forces a new resource to be created.

        > **Granting Admin Consent** To grant admin consent for the service principal to impersonate all users, just omit the `user_object_id` property.
        """
        return pulumi.get(self, "user_object_id")

    @user_object_id.setter
    def user_object_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "user_object_id", value)


class ServicePrincipalDelegatedPermissionGrant(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 claim_values: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 resource_service_principal_object_id: Optional[pulumi.Input[str]] = None,
                 service_principal_object_id: Optional[pulumi.Input[str]] = None,
                 user_object_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Manages a delegated permission grant for a service principal, on behalf of a single user, or all users.

        ## API Permissions

        The following API permissions are required in order to use this resource.

        When authenticated with a service principal, this resource requires the following application role: `Directory.ReadWrite.All`

        When authenticated with a user principal, this resource requires one the following directory role: `Global Administrator`

        ## Example Usage

        *Delegated permission grant for all users*

        ```python
        import pulumi
        import pulumi_azuread as azuread

        well_known = azuread.get_application_published_app_ids()
        msgraph = azuread.ServicePrincipal("msgraph",
            application_id=well_known.result["MicrosoftGraph"],
            use_existing=True)
        example_application = azuread.Application("exampleApplication",
            display_name="example",
            required_resource_accesses=[azuread.ApplicationRequiredResourceAccessArgs(
                resource_app_id=well_known.result["MicrosoftGraph"],
                resource_accesses=[
                    azuread.ApplicationRequiredResourceAccessResourceAccessArgs(
                        id=msgraph.oauth2_permission_scope_ids["openid"],
                        type="Scope",
                    ),
                    azuread.ApplicationRequiredResourceAccessResourceAccessArgs(
                        id=msgraph.oauth2_permission_scope_ids["User.Read"],
                        type="Scope",
                    ),
                ],
            )])
        example_service_principal = azuread.ServicePrincipal("exampleServicePrincipal", application_id=example_application.application_id)
        example_service_principal_delegated_permission_grant = azuread.ServicePrincipalDelegatedPermissionGrant("exampleServicePrincipalDelegatedPermissionGrant",
            service_principal_object_id=example_service_principal.object_id,
            resource_service_principal_object_id=msgraph.object_id,
            claim_values=[
                "openid",
                "User.Read.All",
            ])
        ```

        *Delegated permission grant for a single user*

        ```python
        import pulumi
        import pulumi_azuread as azuread

        well_known = azuread.get_application_published_app_ids()
        msgraph = azuread.ServicePrincipal("msgraph",
            application_id=well_known.result["MicrosoftGraph"],
            use_existing=True)
        example_application = azuread.Application("exampleApplication",
            display_name="example",
            required_resource_accesses=[azuread.ApplicationRequiredResourceAccessArgs(
                resource_app_id=well_known.result["MicrosoftGraph"],
                resource_accesses=[
                    azuread.ApplicationRequiredResourceAccessResourceAccessArgs(
                        id=msgraph.oauth2_permission_scope_ids["openid"],
                        type="Scope",
                    ),
                    azuread.ApplicationRequiredResourceAccessResourceAccessArgs(
                        id=msgraph.oauth2_permission_scope_ids["User.Read"],
                        type="Scope",
                    ),
                ],
            )])
        example_service_principal = azuread.ServicePrincipal("exampleServicePrincipal", application_id=example_application.application_id)
        example_user = azuread.User("exampleUser",
            display_name="J. Doe",
            user_principal_name="jdoe@hashicorp.com",
            mail_nickname="jdoe",
            password="SecretP@sswd99!")
        example_service_principal_delegated_permission_grant = azuread.ServicePrincipalDelegatedPermissionGrant("exampleServicePrincipalDelegatedPermissionGrant",
            service_principal_object_id=example_service_principal.object_id,
            resource_service_principal_object_id=msgraph.object_id,
            claim_values=[
                "openid",
                "User.Read.All",
            ],
            user_object_id=example_user.object_id)
        ```

        ## Import

        Delegated permission grants can be imported using their ID, e.g.

        ```sh
         $ pulumi import azuread:index/servicePrincipalDelegatedPermissionGrant:ServicePrincipalDelegatedPermissionGrant example aaBBcDDeFG6h5JKLMN2PQrrssTTUUvWWxxxxxyyyzzz
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] claim_values: A set of claim values for delegated permission scopes which should be included in access tokens for the resource.
        :param pulumi.Input[str] resource_service_principal_object_id: The object ID of the service principal representing the resource to be accessed. Changing this forces a new resource to be created.
        :param pulumi.Input[str] service_principal_object_id: The object ID of the service principal for which this delegated permission grant should be created. Changing this forces a new resource to be created.
        :param pulumi.Input[str] user_object_id: The object ID of the user on behalf of whom the service principal is authorized to access the resource. When omitted, the delegated permission grant will be consented for all users. Changing this forces a new resource to be created.
               
               > **Granting Admin Consent** To grant admin consent for the service principal to impersonate all users, just omit the `user_object_id` property.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ServicePrincipalDelegatedPermissionGrantArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages a delegated permission grant for a service principal, on behalf of a single user, or all users.

        ## API Permissions

        The following API permissions are required in order to use this resource.

        When authenticated with a service principal, this resource requires the following application role: `Directory.ReadWrite.All`

        When authenticated with a user principal, this resource requires one the following directory role: `Global Administrator`

        ## Example Usage

        *Delegated permission grant for all users*

        ```python
        import pulumi
        import pulumi_azuread as azuread

        well_known = azuread.get_application_published_app_ids()
        msgraph = azuread.ServicePrincipal("msgraph",
            application_id=well_known.result["MicrosoftGraph"],
            use_existing=True)
        example_application = azuread.Application("exampleApplication",
            display_name="example",
            required_resource_accesses=[azuread.ApplicationRequiredResourceAccessArgs(
                resource_app_id=well_known.result["MicrosoftGraph"],
                resource_accesses=[
                    azuread.ApplicationRequiredResourceAccessResourceAccessArgs(
                        id=msgraph.oauth2_permission_scope_ids["openid"],
                        type="Scope",
                    ),
                    azuread.ApplicationRequiredResourceAccessResourceAccessArgs(
                        id=msgraph.oauth2_permission_scope_ids["User.Read"],
                        type="Scope",
                    ),
                ],
            )])
        example_service_principal = azuread.ServicePrincipal("exampleServicePrincipal", application_id=example_application.application_id)
        example_service_principal_delegated_permission_grant = azuread.ServicePrincipalDelegatedPermissionGrant("exampleServicePrincipalDelegatedPermissionGrant",
            service_principal_object_id=example_service_principal.object_id,
            resource_service_principal_object_id=msgraph.object_id,
            claim_values=[
                "openid",
                "User.Read.All",
            ])
        ```

        *Delegated permission grant for a single user*

        ```python
        import pulumi
        import pulumi_azuread as azuread

        well_known = azuread.get_application_published_app_ids()
        msgraph = azuread.ServicePrincipal("msgraph",
            application_id=well_known.result["MicrosoftGraph"],
            use_existing=True)
        example_application = azuread.Application("exampleApplication",
            display_name="example",
            required_resource_accesses=[azuread.ApplicationRequiredResourceAccessArgs(
                resource_app_id=well_known.result["MicrosoftGraph"],
                resource_accesses=[
                    azuread.ApplicationRequiredResourceAccessResourceAccessArgs(
                        id=msgraph.oauth2_permission_scope_ids["openid"],
                        type="Scope",
                    ),
                    azuread.ApplicationRequiredResourceAccessResourceAccessArgs(
                        id=msgraph.oauth2_permission_scope_ids["User.Read"],
                        type="Scope",
                    ),
                ],
            )])
        example_service_principal = azuread.ServicePrincipal("exampleServicePrincipal", application_id=example_application.application_id)
        example_user = azuread.User("exampleUser",
            display_name="J. Doe",
            user_principal_name="jdoe@hashicorp.com",
            mail_nickname="jdoe",
            password="SecretP@sswd99!")
        example_service_principal_delegated_permission_grant = azuread.ServicePrincipalDelegatedPermissionGrant("exampleServicePrincipalDelegatedPermissionGrant",
            service_principal_object_id=example_service_principal.object_id,
            resource_service_principal_object_id=msgraph.object_id,
            claim_values=[
                "openid",
                "User.Read.All",
            ],
            user_object_id=example_user.object_id)
        ```

        ## Import

        Delegated permission grants can be imported using their ID, e.g.

        ```sh
         $ pulumi import azuread:index/servicePrincipalDelegatedPermissionGrant:ServicePrincipalDelegatedPermissionGrant example aaBBcDDeFG6h5JKLMN2PQrrssTTUUvWWxxxxxyyyzzz
        ```

        :param str resource_name: The name of the resource.
        :param ServicePrincipalDelegatedPermissionGrantArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ServicePrincipalDelegatedPermissionGrantArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            kwargs = kwargs or {}
            def _setter(key, value):
                kwargs[key] = value
            ServicePrincipalDelegatedPermissionGrantArgs._configure(_setter, **kwargs)
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 claim_values: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 resource_service_principal_object_id: Optional[pulumi.Input[str]] = None,
                 service_principal_object_id: Optional[pulumi.Input[str]] = None,
                 user_object_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ServicePrincipalDelegatedPermissionGrantArgs.__new__(ServicePrincipalDelegatedPermissionGrantArgs)

            if claim_values is None and not opts.urn:
                raise TypeError("Missing required property 'claim_values'")
            __props__.__dict__["claim_values"] = claim_values
            if resource_service_principal_object_id is None and not opts.urn:
                raise TypeError("Missing required property 'resource_service_principal_object_id'")
            __props__.__dict__["resource_service_principal_object_id"] = resource_service_principal_object_id
            if service_principal_object_id is None and not opts.urn:
                raise TypeError("Missing required property 'service_principal_object_id'")
            __props__.__dict__["service_principal_object_id"] = service_principal_object_id
            __props__.__dict__["user_object_id"] = user_object_id
        super(ServicePrincipalDelegatedPermissionGrant, __self__).__init__(
            'azuread:index/servicePrincipalDelegatedPermissionGrant:ServicePrincipalDelegatedPermissionGrant',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            claim_values: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            resource_service_principal_object_id: Optional[pulumi.Input[str]] = None,
            service_principal_object_id: Optional[pulumi.Input[str]] = None,
            user_object_id: Optional[pulumi.Input[str]] = None) -> 'ServicePrincipalDelegatedPermissionGrant':
        """
        Get an existing ServicePrincipalDelegatedPermissionGrant resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] claim_values: A set of claim values for delegated permission scopes which should be included in access tokens for the resource.
        :param pulumi.Input[str] resource_service_principal_object_id: The object ID of the service principal representing the resource to be accessed. Changing this forces a new resource to be created.
        :param pulumi.Input[str] service_principal_object_id: The object ID of the service principal for which this delegated permission grant should be created. Changing this forces a new resource to be created.
        :param pulumi.Input[str] user_object_id: The object ID of the user on behalf of whom the service principal is authorized to access the resource. When omitted, the delegated permission grant will be consented for all users. Changing this forces a new resource to be created.
               
               > **Granting Admin Consent** To grant admin consent for the service principal to impersonate all users, just omit the `user_object_id` property.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ServicePrincipalDelegatedPermissionGrantState.__new__(_ServicePrincipalDelegatedPermissionGrantState)

        __props__.__dict__["claim_values"] = claim_values
        __props__.__dict__["resource_service_principal_object_id"] = resource_service_principal_object_id
        __props__.__dict__["service_principal_object_id"] = service_principal_object_id
        __props__.__dict__["user_object_id"] = user_object_id
        return ServicePrincipalDelegatedPermissionGrant(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="claimValues")
    def claim_values(self) -> pulumi.Output[Sequence[str]]:
        """
        A set of claim values for delegated permission scopes which should be included in access tokens for the resource.
        """
        return pulumi.get(self, "claim_values")

    @property
    @pulumi.getter(name="resourceServicePrincipalObjectId")
    def resource_service_principal_object_id(self) -> pulumi.Output[str]:
        """
        The object ID of the service principal representing the resource to be accessed. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "resource_service_principal_object_id")

    @property
    @pulumi.getter(name="servicePrincipalObjectId")
    def service_principal_object_id(self) -> pulumi.Output[str]:
        """
        The object ID of the service principal for which this delegated permission grant should be created. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "service_principal_object_id")

    @property
    @pulumi.getter(name="userObjectId")
    def user_object_id(self) -> pulumi.Output[Optional[str]]:
        """
        The object ID of the user on behalf of whom the service principal is authorized to access the resource. When omitted, the delegated permission grant will be consented for all users. Changing this forces a new resource to be created.

        > **Granting Admin Consent** To grant admin consent for the service principal to impersonate all users, just omit the `user_object_id` property.
        """
        return pulumi.get(self, "user_object_id")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['AccessPackageAssignmentPolicyArgs', 'AccessPackageAssignmentPolicy']

@pulumi.input_type
class AccessPackageAssignmentPolicyArgs:
    def __init__(__self__, *,
                 access_package_id: pulumi.Input[str],
                 description: pulumi.Input[str],
                 display_name: pulumi.Input[str],
                 approval_settings: Optional[pulumi.Input['AccessPackageAssignmentPolicyApprovalSettingsArgs']] = None,
                 assignment_review_settings: Optional[pulumi.Input['AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs']] = None,
                 duration_in_days: Optional[pulumi.Input[int]] = None,
                 expiration_date: Optional[pulumi.Input[str]] = None,
                 extension_enabled: Optional[pulumi.Input[bool]] = None,
                 questions: Optional[pulumi.Input[Sequence[pulumi.Input['AccessPackageAssignmentPolicyQuestionArgs']]]] = None,
                 requestor_settings: Optional[pulumi.Input['AccessPackageAssignmentPolicyRequestorSettingsArgs']] = None):
        """
        The set of arguments for constructing a AccessPackageAssignmentPolicy resource.
        :param pulumi.Input[str] access_package_id: The ID of the access package that will contain the policy.
        :param pulumi.Input[str] description: The description of the policy.
        :param pulumi.Input[str] display_name: The display name of the policy.
        :param pulumi.Input['AccessPackageAssignmentPolicyApprovalSettingsArgs'] approval_settings: An `approval_settings` block to specify whether approvals are required and how they are obtained, as documented below.
        :param pulumi.Input['AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs'] assignment_review_settings: An `assignment_review_settings` block, to specify whether assignment review is needed and how it is conducted, as documented below.
        :param pulumi.Input[int] duration_in_days: How many days this assignment is valid for.
        :param pulumi.Input[str] expiration_date: The date that this assignment expires, formatted as an RFC3339 date string in UTC(e.g. 2018-01-01T01:02:03Z).
        :param pulumi.Input[bool] extension_enabled: Whether users will be able to request extension of their access to this package before their access expires.
        :param pulumi.Input[Sequence[pulumi.Input['AccessPackageAssignmentPolicyQuestionArgs']]] questions: One or more `question` blocks for the requestor, as documented below.
        :param pulumi.Input['AccessPackageAssignmentPolicyRequestorSettingsArgs'] requestor_settings: A `requestor_settings` block to configure the users who can request access, as documented below.
        """
        AccessPackageAssignmentPolicyArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            access_package_id=access_package_id,
            description=description,
            display_name=display_name,
            approval_settings=approval_settings,
            assignment_review_settings=assignment_review_settings,
            duration_in_days=duration_in_days,
            expiration_date=expiration_date,
            extension_enabled=extension_enabled,
            questions=questions,
            requestor_settings=requestor_settings,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             access_package_id: pulumi.Input[str],
             description: pulumi.Input[str],
             display_name: pulumi.Input[str],
             approval_settings: Optional[pulumi.Input['AccessPackageAssignmentPolicyApprovalSettingsArgs']] = None,
             assignment_review_settings: Optional[pulumi.Input['AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs']] = None,
             duration_in_days: Optional[pulumi.Input[int]] = None,
             expiration_date: Optional[pulumi.Input[str]] = None,
             extension_enabled: Optional[pulumi.Input[bool]] = None,
             questions: Optional[pulumi.Input[Sequence[pulumi.Input['AccessPackageAssignmentPolicyQuestionArgs']]]] = None,
             requestor_settings: Optional[pulumi.Input['AccessPackageAssignmentPolicyRequestorSettingsArgs']] = None,
             opts: Optional[pulumi.ResourceOptions]=None):
        _setter("access_package_id", access_package_id)
        _setter("description", description)
        _setter("display_name", display_name)
        if approval_settings is not None:
            _setter("approval_settings", approval_settings)
        if assignment_review_settings is not None:
            _setter("assignment_review_settings", assignment_review_settings)
        if duration_in_days is not None:
            _setter("duration_in_days", duration_in_days)
        if expiration_date is not None:
            _setter("expiration_date", expiration_date)
        if extension_enabled is not None:
            _setter("extension_enabled", extension_enabled)
        if questions is not None:
            _setter("questions", questions)
        if requestor_settings is not None:
            _setter("requestor_settings", requestor_settings)

    @property
    @pulumi.getter(name="accessPackageId")
    def access_package_id(self) -> pulumi.Input[str]:
        """
        The ID of the access package that will contain the policy.
        """
        return pulumi.get(self, "access_package_id")

    @access_package_id.setter
    def access_package_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "access_package_id", value)

    @property
    @pulumi.getter
    def description(self) -> pulumi.Input[str]:
        """
        The description of the policy.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: pulumi.Input[str]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> pulumi.Input[str]:
        """
        The display name of the policy.
        """
        return pulumi.get(self, "display_name")

    @display_name.setter
    def display_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "display_name", value)

    @property
    @pulumi.getter(name="approvalSettings")
    def approval_settings(self) -> Optional[pulumi.Input['AccessPackageAssignmentPolicyApprovalSettingsArgs']]:
        """
        An `approval_settings` block to specify whether approvals are required and how they are obtained, as documented below.
        """
        return pulumi.get(self, "approval_settings")

    @approval_settings.setter
    def approval_settings(self, value: Optional[pulumi.Input['AccessPackageAssignmentPolicyApprovalSettingsArgs']]):
        pulumi.set(self, "approval_settings", value)

    @property
    @pulumi.getter(name="assignmentReviewSettings")
    def assignment_review_settings(self) -> Optional[pulumi.Input['AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs']]:
        """
        An `assignment_review_settings` block, to specify whether assignment review is needed and how it is conducted, as documented below.
        """
        return pulumi.get(self, "assignment_review_settings")

    @assignment_review_settings.setter
    def assignment_review_settings(self, value: Optional[pulumi.Input['AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs']]):
        pulumi.set(self, "assignment_review_settings", value)

    @property
    @pulumi.getter(name="durationInDays")
    def duration_in_days(self) -> Optional[pulumi.Input[int]]:
        """
        How many days this assignment is valid for.
        """
        return pulumi.get(self, "duration_in_days")

    @duration_in_days.setter
    def duration_in_days(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "duration_in_days", value)

    @property
    @pulumi.getter(name="expirationDate")
    def expiration_date(self) -> Optional[pulumi.Input[str]]:
        """
        The date that this assignment expires, formatted as an RFC3339 date string in UTC(e.g. 2018-01-01T01:02:03Z).
        """
        return pulumi.get(self, "expiration_date")

    @expiration_date.setter
    def expiration_date(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "expiration_date", value)

    @property
    @pulumi.getter(name="extensionEnabled")
    def extension_enabled(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether users will be able to request extension of their access to this package before their access expires.
        """
        return pulumi.get(self, "extension_enabled")

    @extension_enabled.setter
    def extension_enabled(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "extension_enabled", value)

    @property
    @pulumi.getter
    def questions(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['AccessPackageAssignmentPolicyQuestionArgs']]]]:
        """
        One or more `question` blocks for the requestor, as documented below.
        """
        return pulumi.get(self, "questions")

    @questions.setter
    def questions(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['AccessPackageAssignmentPolicyQuestionArgs']]]]):
        pulumi.set(self, "questions", value)

    @property
    @pulumi.getter(name="requestorSettings")
    def requestor_settings(self) -> Optional[pulumi.Input['AccessPackageAssignmentPolicyRequestorSettingsArgs']]:
        """
        A `requestor_settings` block to configure the users who can request access, as documented below.
        """
        return pulumi.get(self, "requestor_settings")

    @requestor_settings.setter
    def requestor_settings(self, value: Optional[pulumi.Input['AccessPackageAssignmentPolicyRequestorSettingsArgs']]):
        pulumi.set(self, "requestor_settings", value)


@pulumi.input_type
class _AccessPackageAssignmentPolicyState:
    def __init__(__self__, *,
                 access_package_id: Optional[pulumi.Input[str]] = None,
                 approval_settings: Optional[pulumi.Input['AccessPackageAssignmentPolicyApprovalSettingsArgs']] = None,
                 assignment_review_settings: Optional[pulumi.Input['AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs']] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 duration_in_days: Optional[pulumi.Input[int]] = None,
                 expiration_date: Optional[pulumi.Input[str]] = None,
                 extension_enabled: Optional[pulumi.Input[bool]] = None,
                 questions: Optional[pulumi.Input[Sequence[pulumi.Input['AccessPackageAssignmentPolicyQuestionArgs']]]] = None,
                 requestor_settings: Optional[pulumi.Input['AccessPackageAssignmentPolicyRequestorSettingsArgs']] = None):
        """
        Input properties used for looking up and filtering AccessPackageAssignmentPolicy resources.
        :param pulumi.Input[str] access_package_id: The ID of the access package that will contain the policy.
        :param pulumi.Input['AccessPackageAssignmentPolicyApprovalSettingsArgs'] approval_settings: An `approval_settings` block to specify whether approvals are required and how they are obtained, as documented below.
        :param pulumi.Input['AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs'] assignment_review_settings: An `assignment_review_settings` block, to specify whether assignment review is needed and how it is conducted, as documented below.
        :param pulumi.Input[str] description: The description of the policy.
        :param pulumi.Input[str] display_name: The display name of the policy.
        :param pulumi.Input[int] duration_in_days: How many days this assignment is valid for.
        :param pulumi.Input[str] expiration_date: The date that this assignment expires, formatted as an RFC3339 date string in UTC(e.g. 2018-01-01T01:02:03Z).
        :param pulumi.Input[bool] extension_enabled: Whether users will be able to request extension of their access to this package before their access expires.
        :param pulumi.Input[Sequence[pulumi.Input['AccessPackageAssignmentPolicyQuestionArgs']]] questions: One or more `question` blocks for the requestor, as documented below.
        :param pulumi.Input['AccessPackageAssignmentPolicyRequestorSettingsArgs'] requestor_settings: A `requestor_settings` block to configure the users who can request access, as documented below.
        """
        _AccessPackageAssignmentPolicyState._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            access_package_id=access_package_id,
            approval_settings=approval_settings,
            assignment_review_settings=assignment_review_settings,
            description=description,
            display_name=display_name,
            duration_in_days=duration_in_days,
            expiration_date=expiration_date,
            extension_enabled=extension_enabled,
            questions=questions,
            requestor_settings=requestor_settings,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             access_package_id: Optional[pulumi.Input[str]] = None,
             approval_settings: Optional[pulumi.Input['AccessPackageAssignmentPolicyApprovalSettingsArgs']] = None,
             assignment_review_settings: Optional[pulumi.Input['AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs']] = None,
             description: Optional[pulumi.Input[str]] = None,
             display_name: Optional[pulumi.Input[str]] = None,
             duration_in_days: Optional[pulumi.Input[int]] = None,
             expiration_date: Optional[pulumi.Input[str]] = None,
             extension_enabled: Optional[pulumi.Input[bool]] = None,
             questions: Optional[pulumi.Input[Sequence[pulumi.Input['AccessPackageAssignmentPolicyQuestionArgs']]]] = None,
             requestor_settings: Optional[pulumi.Input['AccessPackageAssignmentPolicyRequestorSettingsArgs']] = None,
             opts: Optional[pulumi.ResourceOptions]=None):
        if access_package_id is not None:
            _setter("access_package_id", access_package_id)
        if approval_settings is not None:
            _setter("approval_settings", approval_settings)
        if assignment_review_settings is not None:
            _setter("assignment_review_settings", assignment_review_settings)
        if description is not None:
            _setter("description", description)
        if display_name is not None:
            _setter("display_name", display_name)
        if duration_in_days is not None:
            _setter("duration_in_days", duration_in_days)
        if expiration_date is not None:
            _setter("expiration_date", expiration_date)
        if extension_enabled is not None:
            _setter("extension_enabled", extension_enabled)
        if questions is not None:
            _setter("questions", questions)
        if requestor_settings is not None:
            _setter("requestor_settings", requestor_settings)

    @property
    @pulumi.getter(name="accessPackageId")
    def access_package_id(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the access package that will contain the policy.
        """
        return pulumi.get(self, "access_package_id")

    @access_package_id.setter
    def access_package_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "access_package_id", value)

    @property
    @pulumi.getter(name="approvalSettings")
    def approval_settings(self) -> Optional[pulumi.Input['AccessPackageAssignmentPolicyApprovalSettingsArgs']]:
        """
        An `approval_settings` block to specify whether approvals are required and how they are obtained, as documented below.
        """
        return pulumi.get(self, "approval_settings")

    @approval_settings.setter
    def approval_settings(self, value: Optional[pulumi.Input['AccessPackageAssignmentPolicyApprovalSettingsArgs']]):
        pulumi.set(self, "approval_settings", value)

    @property
    @pulumi.getter(name="assignmentReviewSettings")
    def assignment_review_settings(self) -> Optional[pulumi.Input['AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs']]:
        """
        An `assignment_review_settings` block, to specify whether assignment review is needed and how it is conducted, as documented below.
        """
        return pulumi.get(self, "assignment_review_settings")

    @assignment_review_settings.setter
    def assignment_review_settings(self, value: Optional[pulumi.Input['AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs']]):
        pulumi.set(self, "assignment_review_settings", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        The description of the policy.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> Optional[pulumi.Input[str]]:
        """
        The display name of the policy.
        """
        return pulumi.get(self, "display_name")

    @display_name.setter
    def display_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "display_name", value)

    @property
    @pulumi.getter(name="durationInDays")
    def duration_in_days(self) -> Optional[pulumi.Input[int]]:
        """
        How many days this assignment is valid for.
        """
        return pulumi.get(self, "duration_in_days")

    @duration_in_days.setter
    def duration_in_days(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "duration_in_days", value)

    @property
    @pulumi.getter(name="expirationDate")
    def expiration_date(self) -> Optional[pulumi.Input[str]]:
        """
        The date that this assignment expires, formatted as an RFC3339 date string in UTC(e.g. 2018-01-01T01:02:03Z).
        """
        return pulumi.get(self, "expiration_date")

    @expiration_date.setter
    def expiration_date(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "expiration_date", value)

    @property
    @pulumi.getter(name="extensionEnabled")
    def extension_enabled(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether users will be able to request extension of their access to this package before their access expires.
        """
        return pulumi.get(self, "extension_enabled")

    @extension_enabled.setter
    def extension_enabled(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "extension_enabled", value)

    @property
    @pulumi.getter
    def questions(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['AccessPackageAssignmentPolicyQuestionArgs']]]]:
        """
        One or more `question` blocks for the requestor, as documented below.
        """
        return pulumi.get(self, "questions")

    @questions.setter
    def questions(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['AccessPackageAssignmentPolicyQuestionArgs']]]]):
        pulumi.set(self, "questions", value)

    @property
    @pulumi.getter(name="requestorSettings")
    def requestor_settings(self) -> Optional[pulumi.Input['AccessPackageAssignmentPolicyRequestorSettingsArgs']]:
        """
        A `requestor_settings` block to configure the users who can request access, as documented below.
        """
        return pulumi.get(self, "requestor_settings")

    @requestor_settings.setter
    def requestor_settings(self, value: Optional[pulumi.Input['AccessPackageAssignmentPolicyRequestorSettingsArgs']]):
        pulumi.set(self, "requestor_settings", value)


class AccessPackageAssignmentPolicy(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 access_package_id: Optional[pulumi.Input[str]] = None,
                 approval_settings: Optional[pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyApprovalSettingsArgs']]] = None,
                 assignment_review_settings: Optional[pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs']]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 duration_in_days: Optional[pulumi.Input[int]] = None,
                 expiration_date: Optional[pulumi.Input[str]] = None,
                 extension_enabled: Optional[pulumi.Input[bool]] = None,
                 questions: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyQuestionArgs']]]]] = None,
                 requestor_settings: Optional[pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyRequestorSettingsArgs']]] = None,
                 __props__=None):
        """
        Manages an assignment policy for an access package within Identity Governance in Azure Active Directory.

        ## API Permissions

        The following API permissions are required in order to use this resource.

        When authenticated with a service principal, this resource requires the following application role: `EntitlementManagement.ReadWrite.All`.

        When authenticated with a user principal, this resource requires `Global Administrator` directory role, or one of the `Catalog Owner` and `Access Package Manager` role in Identity Governance.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_azuread as azuread

        example_group = azuread.Group("exampleGroup",
            display_name="group-name",
            security_enabled=True)
        example_access_package_catalog = azuread.AccessPackageCatalog("exampleAccessPackageCatalog",
            display_name="example-catalog",
            description="Example catalog")
        example_access_package = azuread.AccessPackage("exampleAccessPackage",
            catalog_id=example_access_package_catalog.id,
            display_name="access-package",
            description="Access Package")
        example_access_package_assignment_policy = azuread.AccessPackageAssignmentPolicy("exampleAccessPackageAssignmentPolicy",
            access_package_id=example_access_package.id,
            display_name="assignment-policy",
            description="My assignment policy",
            duration_in_days=90,
            requestor_settings=azuread.AccessPackageAssignmentPolicyRequestorSettingsArgs(
                scope_type="AllExistingDirectoryMemberUsers",
            ),
            approval_settings=azuread.AccessPackageAssignmentPolicyApprovalSettingsArgs(
                approval_required=True,
                approval_stages=[azuread.AccessPackageAssignmentPolicyApprovalSettingsApprovalStageArgs(
                    approval_timeout_in_days=14,
                    primary_approvers=[azuread.AccessPackageAssignmentPolicyApprovalSettingsApprovalStagePrimaryApproverArgs(
                        object_id=example_group.object_id,
                        subject_type="groupMembers",
                    )],
                )],
            ),
            assignment_review_settings=azuread.AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs(
                enabled=True,
                review_frequency="weekly",
                duration_in_days=3,
                review_type="Self",
                access_review_timeout_behavior="keepAccess",
            ),
            questions=[azuread.AccessPackageAssignmentPolicyQuestionArgs(
                text=azuread.AccessPackageAssignmentPolicyQuestionTextArgs(
                    default_text="hello, how are you?",
                ),
            )])
        ```

        ## Import

        An access package assignment policy can be imported using the ID, e.g.

        ```sh
         $ pulumi import azuread:index/accessPackageAssignmentPolicy:AccessPackageAssignmentPolicy example 00000000-0000-0000-0000-000000000000
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] access_package_id: The ID of the access package that will contain the policy.
        :param pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyApprovalSettingsArgs']] approval_settings: An `approval_settings` block to specify whether approvals are required and how they are obtained, as documented below.
        :param pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs']] assignment_review_settings: An `assignment_review_settings` block, to specify whether assignment review is needed and how it is conducted, as documented below.
        :param pulumi.Input[str] description: The description of the policy.
        :param pulumi.Input[str] display_name: The display name of the policy.
        :param pulumi.Input[int] duration_in_days: How many days this assignment is valid for.
        :param pulumi.Input[str] expiration_date: The date that this assignment expires, formatted as an RFC3339 date string in UTC(e.g. 2018-01-01T01:02:03Z).
        :param pulumi.Input[bool] extension_enabled: Whether users will be able to request extension of their access to this package before their access expires.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyQuestionArgs']]]] questions: One or more `question` blocks for the requestor, as documented below.
        :param pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyRequestorSettingsArgs']] requestor_settings: A `requestor_settings` block to configure the users who can request access, as documented below.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: AccessPackageAssignmentPolicyArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages an assignment policy for an access package within Identity Governance in Azure Active Directory.

        ## API Permissions

        The following API permissions are required in order to use this resource.

        When authenticated with a service principal, this resource requires the following application role: `EntitlementManagement.ReadWrite.All`.

        When authenticated with a user principal, this resource requires `Global Administrator` directory role, or one of the `Catalog Owner` and `Access Package Manager` role in Identity Governance.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_azuread as azuread

        example_group = azuread.Group("exampleGroup",
            display_name="group-name",
            security_enabled=True)
        example_access_package_catalog = azuread.AccessPackageCatalog("exampleAccessPackageCatalog",
            display_name="example-catalog",
            description="Example catalog")
        example_access_package = azuread.AccessPackage("exampleAccessPackage",
            catalog_id=example_access_package_catalog.id,
            display_name="access-package",
            description="Access Package")
        example_access_package_assignment_policy = azuread.AccessPackageAssignmentPolicy("exampleAccessPackageAssignmentPolicy",
            access_package_id=example_access_package.id,
            display_name="assignment-policy",
            description="My assignment policy",
            duration_in_days=90,
            requestor_settings=azuread.AccessPackageAssignmentPolicyRequestorSettingsArgs(
                scope_type="AllExistingDirectoryMemberUsers",
            ),
            approval_settings=azuread.AccessPackageAssignmentPolicyApprovalSettingsArgs(
                approval_required=True,
                approval_stages=[azuread.AccessPackageAssignmentPolicyApprovalSettingsApprovalStageArgs(
                    approval_timeout_in_days=14,
                    primary_approvers=[azuread.AccessPackageAssignmentPolicyApprovalSettingsApprovalStagePrimaryApproverArgs(
                        object_id=example_group.object_id,
                        subject_type="groupMembers",
                    )],
                )],
            ),
            assignment_review_settings=azuread.AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs(
                enabled=True,
                review_frequency="weekly",
                duration_in_days=3,
                review_type="Self",
                access_review_timeout_behavior="keepAccess",
            ),
            questions=[azuread.AccessPackageAssignmentPolicyQuestionArgs(
                text=azuread.AccessPackageAssignmentPolicyQuestionTextArgs(
                    default_text="hello, how are you?",
                ),
            )])
        ```

        ## Import

        An access package assignment policy can be imported using the ID, e.g.

        ```sh
         $ pulumi import azuread:index/accessPackageAssignmentPolicy:AccessPackageAssignmentPolicy example 00000000-0000-0000-0000-000000000000
        ```

        :param str resource_name: The name of the resource.
        :param AccessPackageAssignmentPolicyArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(AccessPackageAssignmentPolicyArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            kwargs = kwargs or {}
            def _setter(key, value):
                kwargs[key] = value
            AccessPackageAssignmentPolicyArgs._configure(_setter, **kwargs)
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 access_package_id: Optional[pulumi.Input[str]] = None,
                 approval_settings: Optional[pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyApprovalSettingsArgs']]] = None,
                 assignment_review_settings: Optional[pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs']]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 duration_in_days: Optional[pulumi.Input[int]] = None,
                 expiration_date: Optional[pulumi.Input[str]] = None,
                 extension_enabled: Optional[pulumi.Input[bool]] = None,
                 questions: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyQuestionArgs']]]]] = None,
                 requestor_settings: Optional[pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyRequestorSettingsArgs']]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = AccessPackageAssignmentPolicyArgs.__new__(AccessPackageAssignmentPolicyArgs)

            if access_package_id is None and not opts.urn:
                raise TypeError("Missing required property 'access_package_id'")
            __props__.__dict__["access_package_id"] = access_package_id
            if approval_settings is not None and not isinstance(approval_settings, AccessPackageAssignmentPolicyApprovalSettingsArgs):
                approval_settings = approval_settings or {}
                def _setter(key, value):
                    approval_settings[key] = value
                AccessPackageAssignmentPolicyApprovalSettingsArgs._configure(_setter, **approval_settings)
            __props__.__dict__["approval_settings"] = approval_settings
            if assignment_review_settings is not None and not isinstance(assignment_review_settings, AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs):
                assignment_review_settings = assignment_review_settings or {}
                def _setter(key, value):
                    assignment_review_settings[key] = value
                AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs._configure(_setter, **assignment_review_settings)
            __props__.__dict__["assignment_review_settings"] = assignment_review_settings
            if description is None and not opts.urn:
                raise TypeError("Missing required property 'description'")
            __props__.__dict__["description"] = description
            if display_name is None and not opts.urn:
                raise TypeError("Missing required property 'display_name'")
            __props__.__dict__["display_name"] = display_name
            __props__.__dict__["duration_in_days"] = duration_in_days
            __props__.__dict__["expiration_date"] = expiration_date
            __props__.__dict__["extension_enabled"] = extension_enabled
            __props__.__dict__["questions"] = questions
            if requestor_settings is not None and not isinstance(requestor_settings, AccessPackageAssignmentPolicyRequestorSettingsArgs):
                requestor_settings = requestor_settings or {}
                def _setter(key, value):
                    requestor_settings[key] = value
                AccessPackageAssignmentPolicyRequestorSettingsArgs._configure(_setter, **requestor_settings)
            __props__.__dict__["requestor_settings"] = requestor_settings
        super(AccessPackageAssignmentPolicy, __self__).__init__(
            'azuread:index/accessPackageAssignmentPolicy:AccessPackageAssignmentPolicy',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            access_package_id: Optional[pulumi.Input[str]] = None,
            approval_settings: Optional[pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyApprovalSettingsArgs']]] = None,
            assignment_review_settings: Optional[pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs']]] = None,
            description: Optional[pulumi.Input[str]] = None,
            display_name: Optional[pulumi.Input[str]] = None,
            duration_in_days: Optional[pulumi.Input[int]] = None,
            expiration_date: Optional[pulumi.Input[str]] = None,
            extension_enabled: Optional[pulumi.Input[bool]] = None,
            questions: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyQuestionArgs']]]]] = None,
            requestor_settings: Optional[pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyRequestorSettingsArgs']]] = None) -> 'AccessPackageAssignmentPolicy':
        """
        Get an existing AccessPackageAssignmentPolicy resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] access_package_id: The ID of the access package that will contain the policy.
        :param pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyApprovalSettingsArgs']] approval_settings: An `approval_settings` block to specify whether approvals are required and how they are obtained, as documented below.
        :param pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyAssignmentReviewSettingsArgs']] assignment_review_settings: An `assignment_review_settings` block, to specify whether assignment review is needed and how it is conducted, as documented below.
        :param pulumi.Input[str] description: The description of the policy.
        :param pulumi.Input[str] display_name: The display name of the policy.
        :param pulumi.Input[int] duration_in_days: How many days this assignment is valid for.
        :param pulumi.Input[str] expiration_date: The date that this assignment expires, formatted as an RFC3339 date string in UTC(e.g. 2018-01-01T01:02:03Z).
        :param pulumi.Input[bool] extension_enabled: Whether users will be able to request extension of their access to this package before their access expires.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyQuestionArgs']]]] questions: One or more `question` blocks for the requestor, as documented below.
        :param pulumi.Input[pulumi.InputType['AccessPackageAssignmentPolicyRequestorSettingsArgs']] requestor_settings: A `requestor_settings` block to configure the users who can request access, as documented below.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _AccessPackageAssignmentPolicyState.__new__(_AccessPackageAssignmentPolicyState)

        __props__.__dict__["access_package_id"] = access_package_id
        __props__.__dict__["approval_settings"] = approval_settings
        __props__.__dict__["assignment_review_settings"] = assignment_review_settings
        __props__.__dict__["description"] = description
        __props__.__dict__["display_name"] = display_name
        __props__.__dict__["duration_in_days"] = duration_in_days
        __props__.__dict__["expiration_date"] = expiration_date
        __props__.__dict__["extension_enabled"] = extension_enabled
        __props__.__dict__["questions"] = questions
        __props__.__dict__["requestor_settings"] = requestor_settings
        return AccessPackageAssignmentPolicy(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="accessPackageId")
    def access_package_id(self) -> pulumi.Output[str]:
        """
        The ID of the access package that will contain the policy.
        """
        return pulumi.get(self, "access_package_id")

    @property
    @pulumi.getter(name="approvalSettings")
    def approval_settings(self) -> pulumi.Output[Optional['outputs.AccessPackageAssignmentPolicyApprovalSettings']]:
        """
        An `approval_settings` block to specify whether approvals are required and how they are obtained, as documented below.
        """
        return pulumi.get(self, "approval_settings")

    @property
    @pulumi.getter(name="assignmentReviewSettings")
    def assignment_review_settings(self) -> pulumi.Output[Optional['outputs.AccessPackageAssignmentPolicyAssignmentReviewSettings']]:
        """
        An `assignment_review_settings` block, to specify whether assignment review is needed and how it is conducted, as documented below.
        """
        return pulumi.get(self, "assignment_review_settings")

    @property
    @pulumi.getter
    def description(self) -> pulumi.Output[str]:
        """
        The description of the policy.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> pulumi.Output[str]:
        """
        The display name of the policy.
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter(name="durationInDays")
    def duration_in_days(self) -> pulumi.Output[Optional[int]]:
        """
        How many days this assignment is valid for.
        """
        return pulumi.get(self, "duration_in_days")

    @property
    @pulumi.getter(name="expirationDate")
    def expiration_date(self) -> pulumi.Output[Optional[str]]:
        """
        The date that this assignment expires, formatted as an RFC3339 date string in UTC(e.g. 2018-01-01T01:02:03Z).
        """
        return pulumi.get(self, "expiration_date")

    @property
    @pulumi.getter(name="extensionEnabled")
    def extension_enabled(self) -> pulumi.Output[Optional[bool]]:
        """
        Whether users will be able to request extension of their access to this package before their access expires.
        """
        return pulumi.get(self, "extension_enabled")

    @property
    @pulumi.getter
    def questions(self) -> pulumi.Output[Optional[Sequence['outputs.AccessPackageAssignmentPolicyQuestion']]]:
        """
        One or more `question` blocks for the requestor, as documented below.
        """
        return pulumi.get(self, "questions")

    @property
    @pulumi.getter(name="requestorSettings")
    def requestor_settings(self) -> pulumi.Output[Optional['outputs.AccessPackageAssignmentPolicyRequestorSettings']]:
        """
        A `requestor_settings` block to configure the users who can request access, as documented below.
        """
        return pulumi.get(self, "requestor_settings")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from . import _utilities
from . import outputs

__all__ = [
    'GetDirectoryRoleTemplatesResult',
    'AwaitableGetDirectoryRoleTemplatesResult',
    'get_directory_role_templates',
    'get_directory_role_templates_output',
]

@pulumi.output_type
class GetDirectoryRoleTemplatesResult:
    """
    A collection of values returned by getDirectoryRoleTemplates.
    """
    def __init__(__self__, id=None, object_ids=None, role_templates=None):
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if object_ids and not isinstance(object_ids, list):
            raise TypeError("Expected argument 'object_ids' to be a list")
        pulumi.set(__self__, "object_ids", object_ids)
        if role_templates and not isinstance(role_templates, list):
            raise TypeError("Expected argument 'role_templates' to be a list")
        pulumi.set(__self__, "role_templates", role_templates)

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="objectIds")
    def object_ids(self) -> Sequence[str]:
        """
        The object IDs of the role templates.
        """
        return pulumi.get(self, "object_ids")

    @property
    @pulumi.getter(name="roleTemplates")
    def role_templates(self) -> Sequence['outputs.GetDirectoryRoleTemplatesRoleTemplateResult']:
        """
        A list of role templates. Each `role_template` object provides the attributes documented below.
        """
        return pulumi.get(self, "role_templates")


class AwaitableGetDirectoryRoleTemplatesResult(GetDirectoryRoleTemplatesResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetDirectoryRoleTemplatesResult(
            id=self.id,
            object_ids=self.object_ids,
            role_templates=self.role_templates)


def get_directory_role_templates(opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetDirectoryRoleTemplatesResult:
    """
    Use this data source to access information about directory role templates within Azure Active Directory.

    ## API Permissions

    The following API permissions are required in order to use this resource.

    When authenticated with a service principal, this resource requires one of the following application roles: `RoleManagement.Read.Directory` or `Directory.Read.All`

    When authenticated with a user principal, this data source does not require any additional roles.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_azuread as azuread

    current = azuread.get_directory_role_templates()
    pulumi.export("roles", current.object_ids)
    ```
    """
    __args__ = dict()
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('azuread:index/getDirectoryRoleTemplates:getDirectoryRoleTemplates', __args__, opts=opts, typ=GetDirectoryRoleTemplatesResult).value

    return AwaitableGetDirectoryRoleTemplatesResult(
        id=pulumi.get(__ret__, 'id'),
        object_ids=pulumi.get(__ret__, 'object_ids'),
        role_templates=pulumi.get(__ret__, 'role_templates'))


@_utilities.lift_output_func(get_directory_role_templates)
def get_directory_role_templates_output(opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetDirectoryRoleTemplatesResult]:
    """
    Use this data source to access information about directory role templates within Azure Active Directory.

    ## API Permissions

    The following API permissions are required in order to use this resource.

    When authenticated with a service principal, this resource requires one of the following application roles: `RoleManagement.Read.Directory` or `Directory.Read.All`

    When authenticated with a user principal, this data source does not require any additional roles.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_azuread as azuread

    current = azuread.get_directory_role_templates()
    pulumi.export("roles", current.object_ids)
    ```
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = ['ServicePrincipalClaimsMappingPolicyAssignmentArgs', 'ServicePrincipalClaimsMappingPolicyAssignment']

@pulumi.input_type
class ServicePrincipalClaimsMappingPolicyAssignmentArgs:
    def __init__(__self__, *,
                 claims_mapping_policy_id: pulumi.Input[str],
                 service_principal_id: pulumi.Input[str]):
        """
        The set of arguments for constructing a ServicePrincipalClaimsMappingPolicyAssignment resource.
        :param pulumi.Input[str] claims_mapping_policy_id: The ID of the claims mapping policy to assign.
        :param pulumi.Input[str] service_principal_id: The object ID of the service principal for the policy assignment.
        """
        ServicePrincipalClaimsMappingPolicyAssignmentArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            claims_mapping_policy_id=claims_mapping_policy_id,
            service_principal_id=service_principal_id,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             claims_mapping_policy_id: pulumi.Input[str],
             service_principal_id: pulumi.Input[str],
             opts: Optional[pulumi.ResourceOptions]=None):
        _setter("claims_mapping_policy_id", claims_mapping_policy_id)
        _setter("service_principal_id", service_principal_id)

    @property
    @pulumi.getter(name="claimsMappingPolicyId")
    def claims_mapping_policy_id(self) -> pulumi.Input[str]:
        """
        The ID of the claims mapping policy to assign.
        """
        return pulumi.get(self, "claims_mapping_policy_id")

    @claims_mapping_policy_id.setter
    def claims_mapping_policy_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "claims_mapping_policy_id", value)

    @property
    @pulumi.getter(name="servicePrincipalId")
    def service_principal_id(self) -> pulumi.Input[str]:
        """
        The object ID of the service principal for the policy assignment.
        """
        return pulumi.get(self, "service_principal_id")

    @service_principal_id.setter
    def service_principal_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "service_principal_id", value)


@pulumi.input_type
class _ServicePrincipalClaimsMappingPolicyAssignmentState:
    def __init__(__self__, *,
                 claims_mapping_policy_id: Optional[pulumi.Input[str]] = None,
                 service_principal_id: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering ServicePrincipalClaimsMappingPolicyAssignment resources.
        :param pulumi.Input[str] claims_mapping_policy_id: The ID of the claims mapping policy to assign.
        :param pulumi.Input[str] service_principal_id: The object ID of the service principal for the policy assignment.
        """
        _ServicePrincipalClaimsMappingPolicyAssignmentState._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            claims_mapping_policy_id=claims_mapping_policy_id,
            service_principal_id=service_principal_id,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             claims_mapping_policy_id: Optional[pulumi.Input[str]] = None,
             service_principal_id: Optional[pulumi.Input[str]] = None,
             opts: Optional[pulumi.ResourceOptions]=None):
        if claims_mapping_policy_id is not None:
            _setter("claims_mapping_policy_id", claims_mapping_policy_id)
        if service_principal_id is not None:
            _setter("service_principal_id", service_principal_id)

    @property
    @pulumi.getter(name="claimsMappingPolicyId")
    def claims_mapping_policy_id(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the claims mapping policy to assign.
        """
        return pulumi.get(self, "claims_mapping_policy_id")

    @claims_mapping_policy_id.setter
    def claims_mapping_policy_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "claims_mapping_policy_id", value)

    @property
    @pulumi.getter(name="servicePrincipalId")
    def service_principal_id(self) -> Optional[pulumi.Input[str]]:
        """
        The object ID of the service principal for the policy assignment.
        """
        return pulumi.get(self, "service_principal_id")

    @service_principal_id.setter
    def service_principal_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "service_principal_id", value)


class ServicePrincipalClaimsMappingPolicyAssignment(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 claims_mapping_policy_id: Optional[pulumi.Input[str]] = None,
                 service_principal_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Manages a Claims Mapping Policy Assignment within Azure Active Directory.

        ## API Permissions

        The following API permissions are required in order to use this resource.

        When authenticated with a service principal, this resource requires the following application roles: `Policy.ReadWrite.ApplicationConfiguration` and `Policy.Read.All`

        When authenticated with a user principal, this resource requires one of the following directory roles: `Application Administrator` or `Global Administrator`

        ## Example Usage

        ```python
        import pulumi
        import pulumi_azuread as azuread

        app = azuread.ServicePrincipalClaimsMappingPolicyAssignment("app",
            claims_mapping_policy_id=azuread_claims_mapping_policy["my_policy"]["id"],
            service_principal_id=azuread_service_principal["my_principal"]["id"])
        ```

        ## Import

        Claims Mapping Policy can be imported using the `id`, in the form `service-principal-uuid/claimsMappingPolicy/claims-mapping-policy-uuid`, e.g

        ```sh
         $ pulumi import azuread:index/servicePrincipalClaimsMappingPolicyAssignment:ServicePrincipalClaimsMappingPolicyAssignment app 00000000-0000-0000-0000-000000000000/claimsMappingPolicy/11111111-0000-0000-0000-000000000000
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] claims_mapping_policy_id: The ID of the claims mapping policy to assign.
        :param pulumi.Input[str] service_principal_id: The object ID of the service principal for the policy assignment.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ServicePrincipalClaimsMappingPolicyAssignmentArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages a Claims Mapping Policy Assignment within Azure Active Directory.

        ## API Permissions

        The following API permissions are required in order to use this resource.

        When authenticated with a service principal, this resource requires the following application roles: `Policy.ReadWrite.ApplicationConfiguration` and `Policy.Read.All`

        When authenticated with a user principal, this resource requires one of the following directory roles: `Application Administrator` or `Global Administrator`

        ## Example Usage

        ```python
        import pulumi
        import pulumi_azuread as azuread

        app = azuread.ServicePrincipalClaimsMappingPolicyAssignment("app",
            claims_mapping_policy_id=azuread_claims_mapping_policy["my_policy"]["id"],
            service_principal_id=azuread_service_principal["my_principal"]["id"])
        ```

        ## Import

        Claims Mapping Policy can be imported using the `id`, in the form `service-principal-uuid/claimsMappingPolicy/claims-mapping-policy-uuid`, e.g

        ```sh
         $ pulumi import azuread:index/servicePrincipalClaimsMappingPolicyAssignment:ServicePrincipalClaimsMappingPolicyAssignment app 00000000-0000-0000-0000-000000000000/claimsMappingPolicy/11111111-0000-0000-0000-000000000000
        ```

        :param str resource_name: The name of the resource.
        :param ServicePrincipalClaimsMappingPolicyAssignmentArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ServicePrincipalClaimsMappingPolicyAssignmentArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            kwargs = kwargs or {}
            def _setter(key, value):
                kwargs[key] = value
            ServicePrincipalClaimsMappingPolicyAssignmentArgs._configure(_setter, **kwargs)
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 claims_mapping_policy_id: Optional[pulumi.Input[str]] = None,
                 service_principal_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ServicePrincipalClaimsMappingPolicyAssignmentArgs.__new__(ServicePrincipalClaimsMappingPolicyAssignmentArgs)

            if claims_mapping_policy_id is None and not opts.urn:
                raise TypeError("Missing required property 'claims_mapping_policy_id'")
            __props__.__dict__["claims_mapping_policy_id"] = claims_mapping_policy_id
            if service_principal_id is None and not opts.urn:
                raise TypeError("Missing required property 'service_principal_id'")
            __props__.__dict__["service_principal_id"] = service_principal_id
        super(ServicePrincipalClaimsMappingPolicyAssignment, __self__).__init__(
            'azuread:index/servicePrincipalClaimsMappingPolicyAssignment:ServicePrincipalClaimsMappingPolicyAssignment',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            claims_mapping_policy_id: Optional[pulumi.Input[str]] = None,
            service_principal_id: Optional[pulumi.Input[str]] = None) -> 'ServicePrincipalClaimsMappingPolicyAssignment':
        """
        Get an existing ServicePrincipalClaimsMappingPolicyAssignment resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] claims_mapping_policy_id: The ID of the claims mapping policy to assign.
        :param pulumi.Input[str] service_principal_id: The object ID of the service principal for the policy assignment.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ServicePrincipalClaimsMappingPolicyAssignmentState.__new__(_ServicePrincipalClaimsMappingPolicyAssignmentState)

        __props__.__dict__["claims_mapping_policy_id"] = claims_mapping_policy_id
        __props__.__dict__["service_principal_id"] = service_principal_id
        return ServicePrincipalClaimsMappingPolicyAssignment(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="claimsMappingPolicyId")
    def claims_mapping_policy_id(self) -> pulumi.Output[str]:
        """
        The ID of the claims mapping policy to assign.
        """
        return pulumi.get(self, "claims_mapping_policy_id")

    @property
    @pulumi.getter(name="servicePrincipalId")
    def service_principal_id(self) -> pulumi.Output[str]:
        """
        The object ID of the service principal for the policy assignment.
        """
        return pulumi.get(self, "service_principal_id")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['CustomDirectoryRoleArgs', 'CustomDirectoryRole']

@pulumi.input_type
class CustomDirectoryRoleArgs:
    def __init__(__self__, *,
                 display_name: pulumi.Input[str],
                 enabled: pulumi.Input[bool],
                 permissions: pulumi.Input[Sequence[pulumi.Input['CustomDirectoryRolePermissionArgs']]],
                 version: pulumi.Input[str],
                 description: Optional[pulumi.Input[str]] = None,
                 template_id: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a CustomDirectoryRole resource.
        :param pulumi.Input[str] display_name: The display name of the custom directory role.
        :param pulumi.Input[bool] enabled: Indicates whether the role is enabled for assignment.
        :param pulumi.Input[Sequence[pulumi.Input['CustomDirectoryRolePermissionArgs']]] permissions: A collection of `permissions` blocks as documented below.
        :param pulumi.Input[str] version: The version of the role definition. This can be any arbitrary string between 1-128 characters.
        :param pulumi.Input[str] description: The description of the custom directory role.
        :param pulumi.Input[str] template_id: Custom template identifier that is typically used if one needs an identifier to be the same across different directories. Changing this forces a new resource to be created.
        """
        CustomDirectoryRoleArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            display_name=display_name,
            enabled=enabled,
            permissions=permissions,
            version=version,
            description=description,
            template_id=template_id,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             display_name: pulumi.Input[str],
             enabled: pulumi.Input[bool],
             permissions: pulumi.Input[Sequence[pulumi.Input['CustomDirectoryRolePermissionArgs']]],
             version: pulumi.Input[str],
             description: Optional[pulumi.Input[str]] = None,
             template_id: Optional[pulumi.Input[str]] = None,
             opts: Optional[pulumi.ResourceOptions]=None):
        _setter("display_name", display_name)
        _setter("enabled", enabled)
        _setter("permissions", permissions)
        _setter("version", version)
        if description is not None:
            _setter("description", description)
        if template_id is not None:
            _setter("template_id", template_id)

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> pulumi.Input[str]:
        """
        The display name of the custom directory role.
        """
        return pulumi.get(self, "display_name")

    @display_name.setter
    def display_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "display_name", value)

    @property
    @pulumi.getter
    def enabled(self) -> pulumi.Input[bool]:
        """
        Indicates whether the role is enabled for assignment.
        """
        return pulumi.get(self, "enabled")

    @enabled.setter
    def enabled(self, value: pulumi.Input[bool]):
        pulumi.set(self, "enabled", value)

    @property
    @pulumi.getter
    def permissions(self) -> pulumi.Input[Sequence[pulumi.Input['CustomDirectoryRolePermissionArgs']]]:
        """
        A collection of `permissions` blocks as documented below.
        """
        return pulumi.get(self, "permissions")

    @permissions.setter
    def permissions(self, value: pulumi.Input[Sequence[pulumi.Input['CustomDirectoryRolePermissionArgs']]]):
        pulumi.set(self, "permissions", value)

    @property
    @pulumi.getter
    def version(self) -> pulumi.Input[str]:
        """
        The version of the role definition. This can be any arbitrary string between 1-128 characters.
        """
        return pulumi.get(self, "version")

    @version.setter
    def version(self, value: pulumi.Input[str]):
        pulumi.set(self, "version", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        The description of the custom directory role.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter(name="templateId")
    def template_id(self) -> Optional[pulumi.Input[str]]:
        """
        Custom template identifier that is typically used if one needs an identifier to be the same across different directories. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "template_id")

    @template_id.setter
    def template_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "template_id", value)


@pulumi.input_type
class _CustomDirectoryRoleState:
    def __init__(__self__, *,
                 description: Optional[pulumi.Input[str]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 object_id: Optional[pulumi.Input[str]] = None,
                 permissions: Optional[pulumi.Input[Sequence[pulumi.Input['CustomDirectoryRolePermissionArgs']]]] = None,
                 template_id: Optional[pulumi.Input[str]] = None,
                 version: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering CustomDirectoryRole resources.
        :param pulumi.Input[str] description: The description of the custom directory role.
        :param pulumi.Input[str] display_name: The display name of the custom directory role.
        :param pulumi.Input[bool] enabled: Indicates whether the role is enabled for assignment.
        :param pulumi.Input[str] object_id: The object ID of the custom directory role.
        :param pulumi.Input[Sequence[pulumi.Input['CustomDirectoryRolePermissionArgs']]] permissions: A collection of `permissions` blocks as documented below.
        :param pulumi.Input[str] template_id: Custom template identifier that is typically used if one needs an identifier to be the same across different directories. Changing this forces a new resource to be created.
        :param pulumi.Input[str] version: The version of the role definition. This can be any arbitrary string between 1-128 characters.
        """
        _CustomDirectoryRoleState._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            description=description,
            display_name=display_name,
            enabled=enabled,
            object_id=object_id,
            permissions=permissions,
            template_id=template_id,
            version=version,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             description: Optional[pulumi.Input[str]] = None,
             display_name: Optional[pulumi.Input[str]] = None,
             enabled: Optional[pulumi.Input[bool]] = None,
             object_id: Optional[pulumi.Input[str]] = None,
             permissions: Optional[pulumi.Input[Sequence[pulumi.Input['CustomDirectoryRolePermissionArgs']]]] = None,
             template_id: Optional[pulumi.Input[str]] = None,
             version: Optional[pulumi.Input[str]] = None,
             opts: Optional[pulumi.ResourceOptions]=None):
        if description is not None:
            _setter("description", description)
        if display_name is not None:
            _setter("display_name", display_name)
        if enabled is not None:
            _setter("enabled", enabled)
        if object_id is not None:
            _setter("object_id", object_id)
        if permissions is not None:
            _setter("permissions", permissions)
        if template_id is not None:
            _setter("template_id", template_id)
        if version is not None:
            _setter("version", version)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        The description of the custom directory role.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> Optional[pulumi.Input[str]]:
        """
        The display name of the custom directory role.
        """
        return pulumi.get(self, "display_name")

    @display_name.setter
    def display_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "display_name", value)

    @property
    @pulumi.getter
    def enabled(self) -> Optional[pulumi.Input[bool]]:
        """
        Indicates whether the role is enabled for assignment.
        """
        return pulumi.get(self, "enabled")

    @enabled.setter
    def enabled(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "enabled", value)

    @property
    @pulumi.getter(name="objectId")
    def object_id(self) -> Optional[pulumi.Input[str]]:
        """
        The object ID of the custom directory role.
        """
        return pulumi.get(self, "object_id")

    @object_id.setter
    def object_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "object_id", value)

    @property
    @pulumi.getter
    def permissions(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['CustomDirectoryRolePermissionArgs']]]]:
        """
        A collection of `permissions` blocks as documented below.
        """
        return pulumi.get(self, "permissions")

    @permissions.setter
    def permissions(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['CustomDirectoryRolePermissionArgs']]]]):
        pulumi.set(self, "permissions", value)

    @property
    @pulumi.getter(name="templateId")
    def template_id(self) -> Optional[pulumi.Input[str]]:
        """
        Custom template identifier that is typically used if one needs an identifier to be the same across different directories. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "template_id")

    @template_id.setter
    def template_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "template_id", value)

    @property
    @pulumi.getter
    def version(self) -> Optional[pulumi.Input[str]]:
        """
        The version of the role definition. This can be any arbitrary string between 1-128 characters.
        """
        return pulumi.get(self, "version")

    @version.setter
    def version(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "version", value)


class CustomDirectoryRole(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 permissions: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['CustomDirectoryRolePermissionArgs']]]]] = None,
                 template_id: Optional[pulumi.Input[str]] = None,
                 version: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Manages a Custom Directory Role within Azure Active Directory.

        This resource is for managing custom directory roles. For management of built-in roles, see the DirectoryRole resource.

        ## API Permissions

        The following API permissions are required in order to use this resource.

        When authenticated with a service principal, this resource requires one of the following application roles: `RoleManagement.ReadWrite.Directory` or `Directory.ReadWrite.All`

        When authenticated with a user principal, this resource requires one of the following directory roles: `Privileged Role Administrator` or `Global Administrator`

        ## Example Usage

        ```python
        import pulumi
        import pulumi_azuread as azuread

        example = azuread.CustomDirectoryRole("example",
            description="Allows reading applications and updating groups",
            display_name="My Custom Role",
            enabled=True,
            permissions=[
                azuread.CustomDirectoryRolePermissionArgs(
                    allowed_resource_actions=[
                        "microsoft.directory/applications/basic/update",
                        "microsoft.directory/applications/create",
                        "microsoft.directory/applications/standard/read",
                    ],
                ),
                azuread.CustomDirectoryRolePermissionArgs(
                    allowed_resource_actions=[
                        "microsoft.directory/groups/allProperties/read",
                        "microsoft.directory/groups/allProperties/read",
                        "microsoft.directory/groups/basic/update",
                        "microsoft.directory/groups/create",
                        "microsoft.directory/groups/delete",
                    ],
                ),
            ],
            version="1.0")
        ```

        ## Import

        This resource does not support importing.

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: The description of the custom directory role.
        :param pulumi.Input[str] display_name: The display name of the custom directory role.
        :param pulumi.Input[bool] enabled: Indicates whether the role is enabled for assignment.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['CustomDirectoryRolePermissionArgs']]]] permissions: A collection of `permissions` blocks as documented below.
        :param pulumi.Input[str] template_id: Custom template identifier that is typically used if one needs an identifier to be the same across different directories. Changing this forces a new resource to be created.
        :param pulumi.Input[str] version: The version of the role definition. This can be any arbitrary string between 1-128 characters.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: CustomDirectoryRoleArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages a Custom Directory Role within Azure Active Directory.

        This resource is for managing custom directory roles. For management of built-in roles, see the DirectoryRole resource.

        ## API Permissions

        The following API permissions are required in order to use this resource.

        When authenticated with a service principal, this resource requires one of the following application roles: `RoleManagement.ReadWrite.Directory` or `Directory.ReadWrite.All`

        When authenticated with a user principal, this resource requires one of the following directory roles: `Privileged Role Administrator` or `Global Administrator`

        ## Example Usage

        ```python
        import pulumi
        import pulumi_azuread as azuread

        example = azuread.CustomDirectoryRole("example",
            description="Allows reading applications and updating groups",
            display_name="My Custom Role",
            enabled=True,
            permissions=[
                azuread.CustomDirectoryRolePermissionArgs(
                    allowed_resource_actions=[
                        "microsoft.directory/applications/basic/update",
                        "microsoft.directory/applications/create",
                        "microsoft.directory/applications/standard/read",
                    ],
                ),
                azuread.CustomDirectoryRolePermissionArgs(
                    allowed_resource_actions=[
                        "microsoft.directory/groups/allProperties/read",
                        "microsoft.directory/groups/allProperties/read",
                        "microsoft.directory/groups/basic/update",
                        "microsoft.directory/groups/create",
                        "microsoft.directory/groups/delete",
                    ],
                ),
            ],
            version="1.0")
        ```

        ## Import

        This resource does not support importing.

        :param str resource_name: The name of the resource.
        :param CustomDirectoryRoleArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(CustomDirectoryRoleArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            kwargs = kwargs or {}
            def _setter(key, value):
                kwargs[key] = value
            CustomDirectoryRoleArgs._configure(_setter, **kwargs)
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 permissions: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['CustomDirectoryRolePermissionArgs']]]]] = None,
                 template_id: Optional[pulumi.Input[str]] = None,
                 version: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = CustomDirectoryRoleArgs.__new__(CustomDirectoryRoleArgs)

            __props__.__dict__["description"] = description
            if display_name is None and not opts.urn:
                raise TypeError("Missing required property 'display_name'")
            __props__.__dict__["display_name"] = display_name
            if enabled is None and not opts.urn:
                raise TypeError("Missing required property 'enabled'")
            __props__.__dict__["enabled"] = enabled
            if permissions is None and not opts.urn:
                raise TypeError("Missing required property 'permissions'")
            __props__.__dict__["permissions"] = permissions
            __props__.__dict__["template_id"] = template_id
            if version is None and not opts.urn:
                raise TypeError("Missing required property 'version'")
            __props__.__dict__["version"] = version
            __props__.__dict__["object_id"] = None
        super(CustomDirectoryRole, __self__).__init__(
            'azuread:index/customDirectoryRole:CustomDirectoryRole',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            description: Optional[pulumi.Input[str]] = None,
            display_name: Optional[pulumi.Input[str]] = None,
            enabled: Optional[pulumi.Input[bool]] = None,
            object_id: Optional[pulumi.Input[str]] = None,
            permissions: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['CustomDirectoryRolePermissionArgs']]]]] = None,
            template_id: Optional[pulumi.Input[str]] = None,
            version: Optional[pulumi.Input[str]] = None) -> 'CustomDirectoryRole':
        """
        Get an existing CustomDirectoryRole resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: The description of the custom directory role.
        :param pulumi.Input[str] display_name: The display name of the custom directory role.
        :param pulumi.Input[bool] enabled: Indicates whether the role is enabled for assignment.
        :param pulumi.Input[str] object_id: The object ID of the custom directory role.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['CustomDirectoryRolePermissionArgs']]]] permissions: A collection of `permissions` blocks as documented below.
        :param pulumi.Input[str] template_id: Custom template identifier that is typically used if one needs an identifier to be the same across different directories. Changing this forces a new resource to be created.
        :param pulumi.Input[str] version: The version of the role definition. This can be any arbitrary string between 1-128 characters.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _CustomDirectoryRoleState.__new__(_CustomDirectoryRoleState)

        __props__.__dict__["description"] = description
        __props__.__dict__["display_name"] = display_name
        __props__.__dict__["enabled"] = enabled
        __props__.__dict__["object_id"] = object_id
        __props__.__dict__["permissions"] = permissions
        __props__.__dict__["template_id"] = template_id
        __props__.__dict__["version"] = version
        return CustomDirectoryRole(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def description(self) -> pulumi.Output[Optional[str]]:
        """
        The description of the custom directory role.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> pulumi.Output[str]:
        """
        The display name of the custom directory role.
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter
    def enabled(self) -> pulumi.Output[bool]:
        """
        Indicates whether the role is enabled for assignment.
        """
        return pulumi.get(self, "enabled")

    @property
    @pulumi.getter(name="objectId")
    def object_id(self) -> pulumi.Output[str]:
        """
        The object ID of the custom directory role.
        """
        return pulumi.get(self, "object_id")

    @property
    @pulumi.getter
    def permissions(self) -> pulumi.Output[Sequence['outputs.CustomDirectoryRolePermission']]:
        """
        A collection of `permissions` blocks as documented below.
        """
        return pulumi.get(self, "permissions")

    @property
    @pulumi.getter(name="templateId")
    def template_id(self) -> pulumi.Output[str]:
        """
        Custom template identifier that is typically used if one needs an identifier to be the same across different directories. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "template_id")

    @property
    @pulumi.getter
    def version(self) -> pulumi.Output[str]:
        """
        The version of the role definition. This can be any arbitrary string between 1-128 characters.
        """
        return pulumi.get(self, "version")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'GetAccessPackageResult',
    'AwaitableGetAccessPackageResult',
    'get_access_package',
    'get_access_package_output',
]

@pulumi.output_type
class GetAccessPackageResult:
    """
    A collection of values returned by getAccessPackage.
    """
    def __init__(__self__, catalog_id=None, description=None, display_name=None, hidden=None, id=None, object_id=None):
        if catalog_id and not isinstance(catalog_id, str):
            raise TypeError("Expected argument 'catalog_id' to be a str")
        pulumi.set(__self__, "catalog_id", catalog_id)
        if description and not isinstance(description, str):
            raise TypeError("Expected argument 'description' to be a str")
        pulumi.set(__self__, "description", description)
        if display_name and not isinstance(display_name, str):
            raise TypeError("Expected argument 'display_name' to be a str")
        pulumi.set(__self__, "display_name", display_name)
        if hidden and not isinstance(hidden, bool):
            raise TypeError("Expected argument 'hidden' to be a bool")
        pulumi.set(__self__, "hidden", hidden)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if object_id and not isinstance(object_id, str):
            raise TypeError("Expected argument 'object_id' to be a str")
        pulumi.set(__self__, "object_id", object_id)

    @property
    @pulumi.getter(name="catalogId")
    def catalog_id(self) -> Optional[str]:
        return pulumi.get(self, "catalog_id")

    @property
    @pulumi.getter
    def description(self) -> str:
        """
        The description of the access package.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> str:
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter
    def hidden(self) -> bool:
        """
        Whether the access package is hidden from the requestor.
        """
        return pulumi.get(self, "hidden")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="objectId")
    def object_id(self) -> str:
        return pulumi.get(self, "object_id")


class AwaitableGetAccessPackageResult(GetAccessPackageResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetAccessPackageResult(
            catalog_id=self.catalog_id,
            description=self.description,
            display_name=self.display_name,
            hidden=self.hidden,
            id=self.id,
            object_id=self.object_id)


def get_access_package(catalog_id: Optional[str] = None,
                       display_name: Optional[str] = None,
                       object_id: Optional[str] = None,
                       opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetAccessPackageResult:
    """
    Use this data source to retrieve information for an existing access package within Identity Governance in Azure Active Directory.

    ## API Permissions

    The following API permissions are required in order to use this data source.

    When authenticated with a service principal, this data source requires one of the following application roles: `EntitlementManagement.Read.All`, or `EntitlementManagement.ReadWrite.All`.

    When authenticated with a user principal, this data source requires one of the following directory roles: `Catalog owner`, `Catalog reader`, `Access package manager`, `Global Reader`, or `Global Administrator`.

    ## Example Usage

    *Look up by ID*

    ```python
    import pulumi
    import pulumi_azuread as azuread

    example = azuread.get_access_package(object_id="00000000-0000-0000-0000-000000000000")
    ```

    *Look up by DisplayName*

    ```python
    import pulumi
    import pulumi_azuread as azuread

    example = azuread.get_access_package(catalog_id="00000000-0000-0000-0000-000000000000",
        display_name="My access package Catalog")
    ```


    :param str catalog_id: The ID of the Catalog this access package is in.
    :param str display_name: The display name of the access package.
    :param str object_id: The ID of this access package.
           
           > Either `object_id`, or both `catalog_id` and `display_name`, must be specified.
    """
    __args__ = dict()
    __args__['catalogId'] = catalog_id
    __args__['displayName'] = display_name
    __args__['objectId'] = object_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('azuread:index/getAccessPackage:getAccessPackage', __args__, opts=opts, typ=GetAccessPackageResult).value

    return AwaitableGetAccessPackageResult(
        catalog_id=pulumi.get(__ret__, 'catalog_id'),
        description=pulumi.get(__ret__, 'description'),
        display_name=pulumi.get(__ret__, 'display_name'),
        hidden=pulumi.get(__ret__, 'hidden'),
        id=pulumi.get(__ret__, 'id'),
        object_id=pulumi.get(__ret__, 'object_id'))


@_utilities.lift_output_func(get_access_package)
def get_access_package_output(catalog_id: Optional[pulumi.Input[Optional[str]]] = None,
                              display_name: Optional[pulumi.Input[Optional[str]]] = None,
                              object_id: Optional[pulumi.Input[Optional[str]]] = None,
                              opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetAccessPackageResult]:
    """
    Use this data source to retrieve information for an existing access package within Identity Governance in Azure Active Directory.

    ## API Permissions

    The following API permissions are required in order to use this data source.

    When authenticated with a service principal, this data source requires one of the following application roles: `EntitlementManagement.Read.All`, or `EntitlementManagement.ReadWrite.All`.

    When authenticated with a user principal, this data source requires one of the following directory roles: `Catalog owner`, `Catalog reader`, `Access package manager`, `Global Reader`, or `Global Administrator`.

    ## Example Usage

    *Look up by ID*

    ```python
    import pulumi
    import pulumi_azuread as azuread

    example = azuread.get_access_package(object_id="00000000-0000-0000-0000-000000000000")
    ```

    *Look up by DisplayName*

    ```python
    import pulumi
    import pulumi_azuread as azuread

    example = azuread.get_access_package(catalog_id="00000000-0000-0000-0000-000000000000",
        display_name="My access package Catalog")
    ```


    :param str catalog_id: The ID of the Catalog this access package is in.
    :param str display_name: The display name of the access package.
    :param str object_id: The ID of this access package.
           
           > Either `object_id`, or both `catalog_id` and `display_name`, must be specified.
    """
    ...
